


/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/


/*=================== D3186 - Pulse Pattern Generator =======================*/
/* LabWindows/CVI 5.0 Instrument Driver										 */
/* Original Release: July/18/2000	( Revision 1.2 )						 */
/* By:				 Rohde & Schwarz, Tektronix, 							 */
/*                   Contact:	ADVANTEST CORPORATION						 */
/*                   			Home Page; http://www.advantest.co.jp		 */
/*                   			e-mail;    info@advantest.co.jp				 */
/*---------------------------------------------------------------------------*/
/* Modification History: 													 */
/*         Revision 1.2 :  July/18/2000										 */
/*																			 */
/*===========================================================================*/
 


#include <formatio.h>
#include <utility.h>
#include <visa.h>
#include <ansi_c.h>
#include "d3186.h"

/*= DEFINES =================================================================*/
#define D3186_REVISION      "Rev 1.2, 7/2000, CVI 5.01" /*  Instrument driver revision */
#define BUFFER_SIZE			512L						/* File I/O buffer size */
 
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

static ViString tb_switch_state[2] = {"OF","ON"};
static ViString tb_pattern_mode_id[3] = {"PRBS","WORD","FRAME"};
static ViString tb_mark_ratio_id[8] = {"0/8","1/8","1/4","1/2",
									   "8/8","7/8","3/4","1/2B"};
static ViString tb_word_polarity_id[2] = {"WPN","WPI"};
static ViString tb_payload_type_id[3] = {"PLW","PLP","CID"};
static ViString tb_frame_polarity_id[2] = {"FPN","FPI"};
static ViString tb_tracking_prefix[4] = {"D","DB","C","CB"};
static ViString tb_output_mode_id[3] = {"AC","M2V","GND"};
static ViString tb_offset_mode_id[3] = {"OFH","OFM","OFL"};
static ViString tb_file_type[2] = {"S","W"};

static ViReal64 tb_outp_amplitude_limits[3][2] = { { 0.5, 2.0 }, { 0.6, 1.0 }, { 0.5, 2.0 } };
static ViReal64 tb_outp_offset_limits[9][2] = { { -2.0, 2.0 }, { -1.0, -0.6 }, { -2.0, 2.0 },
												{ -3.0, 1.75}, { -1.5, -0.9 }, { -3.0, 1.75},
												{ -4.0, 1.5 }, { -2.0, -1.2 }, { -4.0, 1.5 } };

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViStatus CVIFUNC D3186_ibwrt (ViSession instrSession, ViByte buffer[],
                              ViInt16 bufsize, ViUInt32 *count);
ViStatus CVIFUNC D3186_ibrd (ViSession instrSession, ViByte buffer[],
                              ViInt16 bufsize, ViUInt32 *count);
ViBoolean D3186_invalidViBooleanRange (ViBoolean val);
ViBoolean D3186_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean D3186_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean D3186_invalidIntRange (int val, int min, int max);
ViBoolean D3186_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean D3186_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

void CVIFUNC D3186_configure_and_measure (ViUInt32 count_n)
{
	ViSession viG, viT, viRM;
    ViStatus status, waitStatus;
    ViUInt32 i, retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    ViUInt16 stb;
    ViEvent ehnd;
    ViEventType etype;

    if ((status = viOpenDefaultRM(&viRM)) < VI_SUCCESS) {
    	FmtOut("[%i]: the session to the Resource Manager could not be opened.\n", status);
    	return;
    }
    if ((status = viOpen(viRM, "GPIB0::8::INSTR", NULL, 0, &viG)) < VI_SUCCESS) {
    	FmtOut("[%i]: the session to the Device could not be opened.\n", status);
    	return;
    }
    if ((status = viOpen(viRM, "GPIB0::20::INSTR", NULL, 0, &viT)) < VI_SUCCESS) {
    	FmtOut("[%i]: the session to the Device could not be opened.\n", status);
    	return;
    }
	if ((status = D3186_set_pattern_mode (viG, D3186_PATTERN_TYPE_PRBS)) < VI_SUCCESS)
    	FmtOut("[%i]: cannot set pattern mode.\n", status);
	if ((status = D3186_set_prbs_count_n (viG, count_n)) < VI_SUCCESS)
    	FmtOut("[%i]: cannot set PRBS count.\n", status);
	if ((status = D3186_set_pattern_mode (viT, D3186_PATTERN_TYPE_PRBS)) < VI_SUCCESS)
    	FmtOut("[%i]: cannot set pattern mode.\n", status);
	if ((status = D3186_set_prbs_count_n (viT, count_n)) < VI_SUCCESS)
    	FmtOut("[%i]: cannot set PRBS count.\n", status);
        
    Fmt (wrtBuf, "%s<PB?");
    total = NumFmtdBytes();
    if ((status = D3186_ibwrt(viG, wrtBuf, total, &retCnt)) >= VI_SUCCESS &&
   	    (status = D3186_ibrd(viG, wrtBuf, BUFFER_SIZE-1, &retCnt)) >= VI_SUCCESS)
   		FmtOut("%s\n", wrtBuf);

    viClose(viT);
    viClose(viG);
    viClose(viRM);
}

/*===========================================================================*/
/* Function: D3186_set_clock_rate                                            */
/* Purpose:  This function sets the frequency.                               */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_clock_rate (ViSession instrSession,
                                       ViUInt32 clock_rate_khz)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViUInt32Range (clock_rate_khz, 150000, 12000000))
        return VI_ERROR_PARAMETER2;
	
	Fmt (wrtBuf, "%s<CR %f[p3]", (double)clock_rate_khz/1000.);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_pattern_mode                                          */
/* Purpose:  This function sets the pattern mode.                            */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_pattern_mode (ViSession instrSession,
                                         int pattern_mode_id)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (pattern_mode_id, 1, sizeof(tb_pattern_mode_id)/sizeof(*tb_pattern_mode_id)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_pattern_mode_id[pattern_mode_id - 1]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_prbs_count_n                                          */
/* Purpose:  This function sets the step count N of pseudo random pattern.   */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_prbs_count_n (ViSession instrSession,
                                         ViUInt32 count_n)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if ( count_n != 7 &&
		 count_n != 9 &&
		 count_n != 10 &&
		 count_n != 11 &&
		 count_n != 15 &&
		 count_n != 23 &&
		 count_n != 31 )
        return VI_ERROR_PARAMETER2;
	
	Fmt (wrtBuf, "%s<PB %i[w2p0],0", count_n);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_mark_ratio                                            */
/* Purpose:  This function sets the mark ratio.                              */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_mark_ratio (ViSession instrSession, int mark_ratio_id)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (mark_ratio_id, 1, sizeof(tb_mark_ratio_id)/sizeof(*tb_mark_ratio_id)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<MR %s", tb_mark_ratio_id[mark_ratio_id - 1]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_pattern_trigger_address                               */
/* Purpose:  This function sets trigger address of currently selected        */
/*           pattern mode.                                                   */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_pattern_trigger_address (ViSession instrSession,
                                                    ViUInt32 trigger_address)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	Fmt (wrtBuf, "%s<ADR %i", trigger_address);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_prbs_pattern_trigger_addr                             */
/* Purpose:  This function sets trigger address of pseudo random pattern.    */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_prbs_pattern_trigger_addr (ViSession instrSession,
                                                      ViUInt32 prbs_trigger_address)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViUInt32Range (prbs_trigger_address, 0, 134217727))
        return VI_ERROR_PARAMETER2;
	
	Fmt (wrtBuf, "%s<PBTAD %i", prbs_trigger_address);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_alternate_mode                                        */
/* Purpose:  This function sets alternate mode.                              */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_alternate_mode (ViSession instrSession,
                                           int alternate_mode_state)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (alternate_mode_state, 0, 1))
        return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<ALT%s", tb_switch_state[alternate_mode_state]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_select_alternate_pattern                                  */
/* Purpose:  This function sets alternate pattern.                           */
/*===========================================================================*/
ViStatus CVIFUNC D3186_select_alternate_pattern (ViSession instrSession,
                                                 int alternate_pattern_id)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (alternate_pattern_id, D3186_ALTERNATE_PATTERN_A,
													 D3186_ALTERNATE_PATTERN_B))
        return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<ALT%c", alternate_pattern_id == D3186_ALTERNATE_PATTERN_A ? 'A' : 'B');
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_word_pattern_length                                   */
/* Purpose:  This function sets word pattern length (bit count).             */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_word_pattern_length (ViSession instrSession,
                                                ViUInt32 word_pattern_length)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViUInt32Range (word_pattern_length, 1, 8388608))
        return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<BL %i", word_pattern_length);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_word_pattern_trigger_addr                             */
/* Purpose:  This function sets word pattern trigger address.                */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_word_pattern_trigger_addr (ViSession instrSession,
                                                      ViUInt32 word_pattern_trigger_address)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViUInt32Range (word_pattern_trigger_address, 0, 524287))
        return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<WDTAD %i", word_pattern_trigger_address);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_send_word_pattern                                         */
/* Purpose:  This function sends word pattern.                               */
/*===========================================================================*/
ViStatus CVIFUNC D3186_send_word_pattern (ViSession instrSession,
                                          ViUInt32 word_address,
                                          ViUInt32 word_byte_count,
                                          unsigned char words_sent[])
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViUInt32Range (word_address, 0, 524287))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViUInt32Range (word_byte_count, 1, 1048576))
        return VI_ERROR_PARAMETER3;
	if (!words_sent)
        return VI_ERROR_PARAMETER4;
	
    Fmt (wrtBuf, "%s<BIN %i,%i", word_address, word_byte_count);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
    if ((D3186_status = D3186_ibwrt (instrSession, words_sent, word_byte_count,
    							 &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_send_word_alternate_pattern                               */
/* Purpose:  This function sends word alternate pattern                      */
/*===========================================================================*/
ViStatus CVIFUNC D3186_send_word_alternate_pattern (ViSession instrSession,
                                                    int word_alternate_pattern_id,
                                                    ViUInt32 word_address,
                                                    ViUInt32 word_byte_count,
                                                    unsigned char words_sent[])
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (word_alternate_pattern_id, D3186_ALTERNATE_PATTERN_A,
														  D3186_ALTERNATE_PATTERN_B))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViUInt32Range (word_address, 0, 524287))
        return VI_ERROR_PARAMETER3;
	if (D3186_invalidViUInt32Range (word_byte_count, 1, 1048576))
        return VI_ERROR_PARAMETER4;
	if (!words_sent)
        return VI_ERROR_PARAMETER5;
	
    Fmt (wrtBuf, "%s<W%cBIN %i,%i", word_alternate_pattern_id == D3186_ALTERNATE_PATTERN_A ? 'A' : 'B',
    								word_address, word_byte_count);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
    if ((D3186_status = D3186_ibwrt (instrSession, words_sent, word_byte_count,
									 &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_polarity                                              */
/* Purpose:  This function sets word pattern polarity.                       */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_polarity (ViSession instrSession, int polarity_id)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (polarity_id, 1, sizeof(tb_word_polarity_id)/sizeof(*tb_word_polarity_id)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_word_polarity_id[polarity_id - 1]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_payload_type                                          */
/* Purpose:  This function sets payload type.                                */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_payload_type (ViSession instrSession,
                                         int payload_type_id)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (payload_type_id, 1, sizeof(tb_payload_type_id)/sizeof(*tb_payload_type_id)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_payload_type_id[payload_type_id - 1]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_frame_structure                                       */
/* Purpose:  This function sets pattern configuration.                       */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_frame_structure (ViSession instrSession,
                                            ViUInt32 frame_count,
                                            ViUInt32 row_count,
                                            ViUInt32 byte_count_per_row,
                                            ViUInt32 overhead_byte_count,
                                            ViUInt32 cid_bit_count)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViUInt32Range (frame_count, 1, 8192))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViUInt32Range (row_count, 1, 16))
        return VI_ERROR_PARAMETER3;
	if (D3186_invalidViUInt32Range (byte_count_per_row, 40, 32768))
        return VI_ERROR_PARAMETER4;
	if (D3186_invalidViUInt32Range (overhead_byte_count, 1, byte_count_per_row - 4))
        return VI_ERROR_PARAMETER5;
	if (D3186_invalidViUInt32Range (cid_bit_count, 0,
						(byte_count_per_row - overhead_byte_count) * 8 - 1))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<FRSTR %i,%i,%i,%i,%i", frame_count, row_count,
				 byte_count_per_row, overhead_byte_count, cid_bit_count);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_frame_pattern_trigger                                 */
/* Purpose:  This function sets trigger position of frame pattern.           */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_frame_pattern_trigger (ViSession instrSession,
                                                  ViUInt32 frame_number,
                                                  ViUInt32 row_number,
                                                  ViUInt32 byte_number)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViUInt32Range (frame_number, 1, 8192))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViUInt32Range (row_number, 1, 16))
        return VI_ERROR_PARAMETER3;
	if (D3186_invalidViUInt32Range (byte_number, 1, 32767))
        return VI_ERROR_PARAMETER4;

    Fmt (wrtBuf, "%s<FRT %i,%i,%i", frame_number, row_number, byte_number);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_send_frame_pattern                                        */
/* Purpose:  This function sends frame pattern.                              */
/*===========================================================================*/
ViStatus CVIFUNC D3186_send_frame_pattern (ViSession instrSession,
                                           ViUInt32 frame_number,
                                           ViUInt32 row_number,
                                           ViUInt32 byte_number,
                                           ViUInt32 frame_byte_count,
                                           unsigned char frame_pattern_sent[])
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViUInt32Range (frame_number, 1, 8192))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViUInt32Range (row_number, 1, 16))
        return VI_ERROR_PARAMETER3;
	if (D3186_invalidViUInt32Range (byte_number, 1, 32768))
        return VI_ERROR_PARAMETER4;
	if (D3186_invalidViUInt32Range (frame_byte_count, 1, 1048576))
        return VI_ERROR_PARAMETER5;
	if (!frame_pattern_sent)
        return VI_ERROR_PARAMETER6;
	
    Fmt (wrtBuf, "%s<FBIN %i,%i,%i,%i", frame_number, row_number,
    									byte_number, frame_byte_count);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
    D3186_status = D3186_ibwrt (instrSession, frame_pattern_sent,
										  frame_byte_count, &retCnt);
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_send_frame_alternate_pattern                              */
/* Purpose:  This function sends frame alternate pattern.                    */
/*===========================================================================*/
ViStatus CVIFUNC D3186_send_frame_alternate_pattern (ViSession instrSession,
                                                     int frame_alternate_pattern_id,
                                                     ViUInt32 frame_number,
                                                     ViUInt32 row_number,
                                                     ViUInt32 byte_number,
		                                             ViUInt32 frame_byte_count,
                                                     unsigned char frame_pattern_sent[])
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (frame_alternate_pattern_id, D3186_ALTERNATE_PATTERN_A,
														   D3186_ALTERNATE_PATTERN_B))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViUInt32Range (frame_number, 1, 4096))
        return VI_ERROR_PARAMETER3;
	if (D3186_invalidViUInt32Range (row_number, 1, 16))
        return VI_ERROR_PARAMETER4;
	if (D3186_invalidViUInt32Range (byte_number, 1, 32768))
        return VI_ERROR_PARAMETER5;
	if (D3186_invalidViUInt32Range (frame_byte_count, 1, 1048576))
        return VI_ERROR_PARAMETER6;
	if (!frame_pattern_sent)
        return VI_ERROR_PARAMETER7;
	
    Fmt (wrtBuf, "%s<F%cBIN %i,%i,%i,%i",
    					frame_alternate_pattern_id == D3186_ALTERNATE_PATTERN_A ? 'A' : 'B',
						frame_number, row_number, byte_number, frame_byte_count);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
    if ((D3186_status = D3186_ibwrt (instrSession, frame_pattern_sent,
    							 frame_byte_count, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_frame_polarity                                        */
/* Purpose:  This function sets frame pattern polarity.                      */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_frame_polarity (ViSession instrSession,
                                           int polarity_id)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (polarity_id, 1, sizeof(tb_frame_polarity_id)/sizeof(*tb_frame_polarity_id)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_frame_polarity_id[polarity_id - 1]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_error_addition_rate                                   */
/* Purpose:  This function selects error addition rate.                      */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_error_addition_rate (ViSession instrSession,
                                                ViInt16 negative_of_exponent)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;

	if (D3186_invalidViInt16Range (negative_of_exponent, -2, 9) ||
		(negative_of_exponent > 0 && negative_of_exponent < 4))
        return VI_ERROR_PARAMETER2;

	Fmt (wrtBuf, "%s<EAD %c", negative_of_exponent >= 0 ? negative_of_exponent + '0' :
							 (negative_of_exponent == -1 ? 'S' : 'E'));
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;

    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_output_state                                              */
/* Purpose:  This function sets the output state.                            */
/*===========================================================================*/
ViStatus CVIFUNC D3186_output_state (ViSession instrSession, int state)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (state, 0, 1))
        return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<OUT%s", tb_switch_state[state]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_configure_output                                          */
/* Purpose:  This function configures various output settings.               */
/*===========================================================================*/
ViStatus CVIFUNC D3186_configure_output (ViSession instrSession, int output_id,
                                         int output_mode_id, int offset_mode_id,
                                         ViReal64 amplitude_volt_pp,
                                         ViReal64 output_offset_volt,
                                         int adjust_enable)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (output_id, 1, sizeof(tb_tracking_prefix)/sizeof(*tb_tracking_prefix)))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidIntRange (output_mode_id, 1, sizeof(tb_output_mode_id)/sizeof(*tb_output_mode_id)))
        return VI_ERROR_PARAMETER3;
	if (D3186_invalidIntRange (offset_mode_id, 1, sizeof(tb_offset_mode_id)/sizeof(*tb_offset_mode_id)))
        return VI_ERROR_PARAMETER4;
	if (D3186_invalidViReal64Range (amplitude_volt_pp,
						tb_outp_amplitude_limits[output_mode_id - 1][0],
						tb_outp_amplitude_limits[output_mode_id - 1][1]))
        return VI_ERROR_PARAMETER5;
	if (D3186_invalidViReal64Range (output_offset_volt,
						tb_outp_offset_limits[(offset_mode_id - 1)*3 + (output_mode_id - 1)][0],
						tb_outp_offset_limits[(offset_mode_id - 1)*3 + (output_mode_id - 1)][1]))
        return VI_ERROR_PARAMETER6;
	if (D3186_invalidIntRange (adjust_enable, 0, 1))
        return VI_ERROR_PARAMETER7;

	/* tracking function */
    Fmt (wrtBuf, "%s<%cTRK%s", *tb_tracking_prefix[output_id - 1],
    						   tb_switch_state[(output_id & 1) == 1]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    /* output mode */
    Fmt (wrtBuf, "%s<%s%s", tb_tracking_prefix[output_id - 1],
    						tb_output_mode_id[output_mode_id - 1]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    /* offset mode */
    Fmt (wrtBuf, "%s<%s%s", tb_tracking_prefix[output_id - 1],
    						tb_offset_mode_id[offset_mode_id - 1]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    /* output amplitude */
    Fmt (wrtBuf, "%s<%sAMP %f[p2]", tb_tracking_prefix[output_id - 1],
    								amplitude_volt_pp);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    /* output offset */
    Fmt (wrtBuf, "%s<%sOFF %f[p2]", tb_tracking_prefix[output_id - 1],
    								output_offset_volt);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
	/* cross-point adjust */
    Fmt (wrtBuf, "%s<%sCP%s", tb_tracking_prefix[output_id - 1],
    						  tb_switch_state[adjust_enable]);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_configure_DATA_output                                     */
/* Purpose:  This function configures DATA output settings.                  */
/*===========================================================================*/
ViStatus CVIFUNC D3186_configure_DATA_output (ViSession instrSession,
                                              ViReal64 amplitude_volt_pp,
                                              ViReal64 output_offset_volt)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViReal64Range (amplitude_volt_pp, 0.5, 2.0))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViReal64Range (output_offset_volt, -4.0, 2.0))
        return VI_ERROR_PARAMETER3;

    /* output amplitude */
    Fmt (wrtBuf, "%s<DAMP %f[p2]", amplitude_volt_pp);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    /* output offset */
    Fmt (wrtBuf, "%s<DOFF %f[p2]", output_offset_volt);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_configure_DATA_Bar_output                                 */
/* Purpose:  This function configures DATA-Bar output settings.              */
/*===========================================================================*/
ViStatus CVIFUNC D3186_configure_DATA_Bar_output (ViSession instrSession,
                                                  ViReal64 amplitude_volt_pp,
                                                  ViReal64 output_offset_volt)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViReal64Range (amplitude_volt_pp, 0.5, 2.0))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViReal64Range (output_offset_volt, -4.0, 2.0))
        return VI_ERROR_PARAMETER3;

    /* output amplitude */
    Fmt (wrtBuf, "%s<DBAMP %f[p2]", amplitude_volt_pp);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    /* output offset */
    Fmt (wrtBuf, "%s<DBOFF %f[p2]", output_offset_volt);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_configure_CLOCK_output                                    */
/* Purpose:  This function configures CLOCK output settings.                 */
/*===========================================================================*/
ViStatus CVIFUNC D3186_configure_CLOCK_output (ViSession instrSession,
                                               ViReal64 amplitude_volt_pp,
                                               ViReal64 output_offset_volt)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViReal64Range (amplitude_volt_pp, 0.5, 2.0))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViReal64Range (output_offset_volt, -4.0, 2.0))
        return VI_ERROR_PARAMETER3;

    /* output amplitude */
    Fmt (wrtBuf, "%s<CAMP %f[p2]", amplitude_volt_pp);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    /* output offset */
    Fmt (wrtBuf, "%s<COFF %f[p2]", output_offset_volt);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_configure_CLOCK_Bar_output                                */
/* Purpose:  This function configures CLOCK-Bar output settings.             */
/*===========================================================================*/
ViStatus CVIFUNC D3186_configure_CLOCK_Bar_output (ViSession instrSession,
                                                   ViReal64 amplitude_volt_pp,
                                                   ViReal64 output_offset_volt)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViReal64Range (amplitude_volt_pp, 0.5, 2.0))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViReal64Range (output_offset_volt, -4.0, 2.0))
        return VI_ERROR_PARAMETER3;

    /* output amplitude */
    Fmt (wrtBuf, "%s<CBAMP %f[p2]", amplitude_volt_pp);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    /* output offset */
    Fmt (wrtBuf, "%s<CBOFF %f[p2]", output_offset_volt);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_set_clock_delay                                           */
/* Purpose:  This function sets the clock delay.                             */
/*===========================================================================*/
ViStatus CVIFUNC D3186_set_clock_delay (ViSession instrSession,
                                        ViInt32 clock_delay_ps)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidViInt32Range (clock_delay_ps, -400, 400))
        return VI_ERROR_PARAMETER2;
	
	Fmt (wrtBuf, "%s<DLY %i", clock_delay_ps);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_load_file                                                 */
/* Purpose:  This function loads a file.                                     */
/*===========================================================================*/
ViStatus CVIFUNC D3186_load_file (ViSession instrSession, int file_type_id,
                                  ViUInt32 file_number)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (file_type_id, 1, sizeof(tb_file_type)/sizeof(*tb_file_type)))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViUInt32Range (file_number, 0, 99))
        return VI_ERROR_PARAMETER3;
	
	Fmt (wrtBuf, "%s<LOAD %s,%i", tb_file_type[file_type_id - 1], file_number);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*===========================================================================*/
/* Function: D3186_save_file                                                 */
/* Purpose:  This function saves a file.                                     */
/*===========================================================================*/
ViStatus CVIFUNC D3186_save_file (ViSession instrSession, int file_type_id,
                                  ViUInt32 file_number)
{
    ViStatus D3186_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3186_invalidIntRange (file_type_id, 1, sizeof(tb_file_type)/sizeof(*tb_file_type)))
        return VI_ERROR_PARAMETER2;
	if (D3186_invalidViUInt32Range (file_number, 0, 99))
        return VI_ERROR_PARAMETER3;
	
	Fmt (wrtBuf, "%s<SAVE %s,%i", tb_file_type[file_type_id - 1], file_number);
    total = NumFmtdBytes();
    if ((D3186_status = D3186_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3186_status;
        
    return D3186_status;        
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: D3186_ibwrt                                                     */
/* Purpose:  Wrapper for viWrite.                                            */
/*===========================================================================*/
ViStatus CVIFUNC D3186_ibwrt (ViSession instrSession, ViByte buffer[],
                              ViInt16 bufsize, ViUInt32 *count)
{
    if (instrSession)
    	return viWrite (instrSession, buffer, bufsize, count);
    else {
        FmtOut ("%[%i%]: \042%s\042\n", bufsize, buffer);
        return VI_SUCCESS;        
    }
}

/*===========================================================================*/
/* Function: D3186_ibrd                                                      */
/* Purpose:  Wrapper for viRead.                                             */
/*===========================================================================*/
ViStatus CVIFUNC D3186_ibrd (ViSession instrSession, ViByte buffer[],
                              ViInt16 bufsize, ViUInt32 *count)
{
    return instrSession ? viRead(instrSession, buffer, bufsize, count) : -1;
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean D3186_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean D3186_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Integer Value Out Of Range - int                                */
/* Purpose:  This function checks an integer value to see if it              */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean D3186_invalidIntRange (int val, int min, int max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean D3186_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return        */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean D3186_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
